from hexicapi.verinfo import __version__, __title__, __author__, __license__, __copyright__
import os,socket,base64,time,requests
from bottle import route,run,post,request,response,static_file
allowGuest=False
class action:
    def auth(username,password):
        if os.path.exists("users/"+username):
            with open("users/"+username+"/auth") as f:
                f.seek(0)
                if f.read()==password:
                    return True, False
                else:
                    return False,allowGuest
        else:
            if username!="":
                return allowGuest,allowGuest
            else:
                return False,False
    def idname(id):
        with open("users/"+str(id)) as f:
            name = str(f.read())
        return name
    def nameid(name):
        with open("users/"+str(name)+"/id") as f:
            id = int(f.read())
        return id
def redirect(link):
    return '<meta http-equiv="refresh" content="0; URL='+link+'" />'
def header(margin="10%"):
    return """<style>html {font-family: 'Brush Script MT', cursive;} body {margin:0;padding:0;} .nomargin {margin:0;} .ilnomargin {margin:0;display:inline;}</style>
    <header style="background-color:#101010;color:#FFFFFF;margin-bottom:"""+str(margin)+""";;padding:5;">
    <h1 class="ilnomargin">Maseic</h1>
    <p class="ilnomargin">a web service made by Red</p>
    </header>"""
def readHTML(file):
    return static_file(file, root=os.path.dirname(os.path.realpath(__file__)))
def mainPage():
    r = requests.get('https://hexicapi.free.bg/')
    return r.text
def RUN(host,port,serv=True,page=mainPage):
    if not os.path.exists('users'):
        os.mkdir('users')
    @route("/authpage/<file:path>")
    def authpage(file):
        return readHTML('authpage/'+file)
    @route("/script/<script>")
    def get_script(script):
        if os.path.exists("scripts/"+script):
            with open("scripts/"+script) as f:
                return f.read()
        else:
            return 'window.location.replace("http://maseic.strangled.net");'
    @route("/image/<image>")
    def get_image(image):
        if os.path.exists("images/"+image):
            return static_file(image,"images/")
        else:
            return ""
    @route("/")
    def main():
        return page()
    @route("/login/check")
    def check():
        auth = request.get_cookie("auth")
        if auth=="True":
            auth=True
        else:
            auth=False
        username = request.get_cookie("username")
        if username==None:
            username="NOAUTH"
        password = request.get_cookie("password")
        if password==None:
            password="Tk9BVVRI"
        password=password.encode("utf-8")
        password = base64.b64decode(password).decode()
        if os.path.exists("users/"+username):
            with open("users/"+username+"/auth") as f:
                if auth and password==f.read():
                    auth=True
                else:
                    auth=False
        if auth:
            return "yes:"+username+"\n"+redirect("/")
        else:
            return "no:Guest\n"+redirect("/login")
    @route("/login")
    def login():
        return readHTML('authpage/login.html')
    @route("/register")
    def register():
        return readHTML('authpage/register.html')
    @post('/login/proccess')
    def do_login():
        username = request.forms.get('username')
        password = request.forms.get('pass')
        auth = action.auth(username,password)[0]
        if auth:
            response.set_cookie("auth","True")
        else:
            response.set_cookie("auth","False")
        response.set_cookie("username",username)
        epas=base64.b64encode(bytes(password.encode("utf-8")))
        response.set_cookie("password",epas.decode("utf-8"))
        if not auth:
            return redirect("/login?fail=yes")
        else:
            return redirect("/")
    @post('/register/proccess')
    def do_register():
        username = request.forms.get('username')
        password = request.forms.get('pass')
        #print("register",username,password)
        if username=="":
          return redirect("/register?fail=yes")
        if password=="":
          return redirect("/register?fail=yes")
        if os.path.exists("users/"+username):
            auth = action.auth(username,"")[0]
        else:
            #print("make")
            os.mkdir("users/"+username)
            with open("users/"+username+"/id","a+") as f:
                id=0
                while os.path.exists("users/"+str(id)):
                    id+=1
                f.write(str(id))
            with open("users/" + username + "/auth", "a+") as f:
                f.write(password)
            with open("users/"+str(id),"a+") as f:
                f.write(username)
            auth = action.auth(username, password)[0]
        if auth:
            response.set_cookie("auth","True")
        else:
            response.set_cookie("auth","False")
        response.set_cookie("username",username)
        epas=base64.b64encode(bytes(password.encode("utf-8")))
        response.set_cookie("password",epas.decode("utf-8"))
        if not auth:
            return redirect("/register?fail=yes")
        else:
            return redirect("/")
    print("running webserver")
    try:
      run(host=host,port=port,quiet=serv,reloader=not serv)#,server='gunicorn',keyfile='key.pem',certfile='cert.pem')
    except:
      print("an error occured")
    print("what happened here, looks like it stopped...")


if __name__ == '__main__':
    print('RUN("localhost",80,True,mainPage)')



import socket, time
from hexicapi.verinfo import __version__, __title__, __author__, __license__, __copyright__
working = []
def recv_all(the_socket:socket.socket, packet_size=1024):
    while the_socket in working:
        pass
    working.append(the_socket)
    data = the_socket.recv(16).decode('utf-8')
    try:
        data_length = int(data)
    except:
        return data
    data = b''
    while len(data) < data_length:
        data += the_socket.recv(packet_size)
    working.remove(the_socket)
    return data
class data_not_bytes(Exception):
    pass
def send_all(the_socket:socket.socket, data):
    while the_socket in working:
        pass
    working.append(the_socket)
    if not type(data) in [bytes, bytearray]:
        raise data_not_bytes(f"Data is not of type bytes / bytearray, type: {type(data)}")
        return -1
    length = str(len(data))
    while len(length) < 16:
        length = '0' + length
    the_socket.send(length.encode('utf-8'))
    the_socket.send(data)
    working.remove(the_socket)


from hexicapi.verinfo import *
import os
i = 0
loc=""
inf=""
if not os.path.exists(loc+"logs"):
  os.mkdir(loc+"logs")
while True:
  if not os.path.exists(loc+"logs/log_"+str(i)+".txt"):
    inf=loc+"logs/log_"+str(i)+".txt"
    open(inf,"a").close()
    break
  else:
    i+=1
def log(msg):
  f = open(inf,"a+")
  f.write(msg+"\n")
  f.close()


import socket
import traceback
from hexicapi.socketMessage import *
from hexicapi.verinfo import __version__, __title__, __author__, __license__, __copyright__
BUFFER_SIZE = 1024

ip = "localhost"
port = 81
debug = False

functions={ # print(list(functions)) - > shows all available functions
    'connecting':None,
    'connection_fail':None,
    'connection_success':None,
    'authenticating':None,
    'authentication_fail':None,
    'authentication_success':None,
    'handshake':None,
    'disconnect':None,
    'heartbeat':None,
    'heartbeat_error':None
}
def on_calf(f):
    functions[f.__name__] = f

def calf(name):
    f = functions[name]
    if f != None:
        if type(f) == list:
            f[0](f[1])
        else:
            f()

def disconnect_socket(s):
    try:
        send_all(s, 'bye'.encode())
    except:
        pass
    s.close()

def run(app,username,password='',autoauth=True):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    calf('connecting')
    try:
        s.connect((ip,port))
    except:
        calf('connection_fail')
        return
    calf('connection_success')
    send_all(s, "clientGetID".encode("utf-8"))
    id = recv_all(s, BUFFER_SIZE).decode("utf-8")
    calf('handshake')
    if autoauth:
        calf('authenticating')
        try:
            send_all(s, str('auth:' + username + ':' + password + ':' + app).encode())
            auth_result = recv_all(s, BUFFER_SIZE).decode('utf-8')
        except:
            calf('disconnect')
        if auth_result == 'auth-declined' or auth_result == 'guest-declined':
            calf('authentication_fail')
        else:
            calf('authentication_success')
    class Client:
        info = {
            'ip':ip,
            'port':port,
            'socket':s,
            'id':id,
            'username':username,
            'app':app
        }
        def heartbeat(self):
            try:
                send_all(self.info['socket'], "clientGetID".encode("utf-8"))
                id = recv_all(self.info['socket'], BUFFER_SIZE).decode("utf-8")
            except:
                id=''
            if id == self.info['id']:
                calf('heartbeat')
            else:
                calf('heartbeat_error')
            return id
        def disconnect(self):
            disconnect_socket(self.info['socket'])
            calf('disconnect')
        def send(self, message):
            try:
                _ = message.decode()
                try:
                    send_all(self.info['socket'], message)
                except Exception as e:
                    print(e)
                    return False
            except:
                try:
                    send_all(self.info['socket'], message.encode())
                except Exception:
                    if debug:
                        print(traceback.format_exc())
                    try:
                        send_all(self.info['socket'], message)
                    except Exception:
                        if debug:
                            print(traceback.format_exc())
                        return False
            return True
        def receive(self, packet_size = BUFFER_SIZE):
            try:
                m = recv_all(self.info['socket'], packet_size)
            except Exception:
                if debug:
                    print(traceback.format_exc())
                return False
            try:
                return m.decode("utf-8")
            except Exception:
                if debug:
                    print(traceback.format_exc())
                return m
        def auth(self, app, username, password=''):
            calf('authenticating')
            auth_result = 'auth-declined'
            try:
                send_all(s, str('auth:' + username + ':' + password + ':' + app).encode())
                auth_result = recv_all(s, BUFFER_SIZE).decode('utf-8')
            except:
                calf('disconnect')
            if auth_result == 'auth-declined' or auth_result == 'guest-declined':
                calf('authentication_fail')
            else:
                calf('authentication_success')
    return Client()



from setuptools import setup, find_packages
import codecs
import os

VERSION = '1.0.709'
short = 'Tool for making quick servers.'
long = '''API for making servers and clients to connect to those servers.

github: https://github.com/JustRedTTG/hexicapi'''

# Setting up
setup(
    name="hexicapi",
    version=VERSION,
    author="Red",
    author_email="redtonehair@gmail.com",
    description=short,
    long_description_content_type="text/markdown",
    long_description=long,
    packages=['hexicapi'],
    install_requires=['pgerom','pygame','bottle'],
    package_data={'hexicapi': ['authpage/*.html',

                  'authpage/css/*.css',

                  'authpage/fonts/font-awesome-4.7.0/css/*.css',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.ttf',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.otf',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.eot',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.woff',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.woff2',
                  'authpage/fonts/font-awesome-4.7.0/fonts/*.svg',
                  'authpage/fonts/font-awesome-4.7.0/less/*.less',
                  'authpage/fonts/font-awesome-4.7.0/scss/*.scss',

                  'authpage/fonts/iconic/css/*.css',
                  'authpage/fonts/iconic/fonts/*.ttf',
                  'authpage/fonts/iconic/fonts/*.eot',
                  'authpage/fonts/iconic/fonts/*.woff',
                  'authpage/fonts/iconic/fonts/*.woff2',
                  'authpage/fonts/iconic/fonts/*.svg',

                  'authpage/fonts/poppins/*.ttf',

                  'authpage/images/icons/favicon.ico',

                  'authpage/js/*.js',

                  'authpage/vendor/animate/animate.css',
                  'authpage/vendor/animsition/css/*.css',
                  'authpage/vendor/animsition/js/*.js',
                  'authpage/vendor/bootstrap/css/*.css',
                  'authpage/vendor/bootstrap/css/*.map',
                  'authpage/vendor/bootstrap/js/*.js',
                  'authpage/vendor/countdowntime/*.js',
                  'authpage/vendor/css-hamburgers/*.css',
                  'authpage/vendor/daterangepicker/*.css',
                  'authpage/vendor/daterangepicker/*.js',
                  'authpage/vendor/jquery/*.js',
                  'authpage/vendor/perfect-scrollbar/*.css',
                  'authpage/vendor/perfect-scrollbar/*.js',
                  'authpage/vendor/select2/*.css',
                  'authpage/vendor/select2/*.js'
                ]
    },
    keywords=['python']#,
#    classifiers=[
#        "Development Status :: 1 - Planning",
#        "Intended Audience :: Developers",
#        "Programming Language :: Python :: 3",
#        "Operating System :: Unix",
#        "Operating System :: MacOS :: MacOS X",
#        "Operating System :: Microsoft :: Windows",
#    ]
)



import os, sys
import os.path
import socket,threading,traceback
import hexicapi.web as web
from hexicapi.socketMessage import *
from hexicapi.verinfo import __version__, __title__, __author__, __license__, __copyright__
from random import randint
from time import sleep,time
from pgerom import save as pesave
from pgerom import load as peload
logg = None
ltar = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
identifier=0
BUFFER_SIZE = 1024
ip="localhost"
port=81
webport=80
webpage=web.mainPage
#variables
ipbans=[]
connections = []
rooms = []
free = []
#
allowGuest={}
apps=[]
app_disconnect={}
#
console=[["Welcome To The Server!","all"]]

# Make ID for client
def makeID(cs):
    global identifier,ltar,free
    num=None
    if len(free)>0:
        num = free[0]
        id = str(connections[num]["id"].split("'")[0])+"'"
        del free[0]
    else:
        id=str(identifier)+"'"
        identifier+=1
    for r in range(10):
        id+=ltar[randint(0,len(ltar)-1)]
    if num==None:
        num2=len(connections)
    else:
        num2=num
    iden = {"id": id, "socket":cs,"thread":True,"calldelta":time(),"username":"Guest","guest":True,"auth":False,"app":None,"data":None,"die":True,"room":None}
    if num == None:
        connections.append(iden)
    else:
        connections[num] = iden
    return id,num2

class action:
    def auth(username,password,app):
        try:
            ag = allowGuest[app]
        except:
            ag=False
        if os.path.exists("users/"+username):
            with open("users/"+username+"/auth") as f:
                f.seek(0)
                if f.read()==password:
                    return True, False
                else:
                    return False,ag
        else:
            if username!="":
                return ag,ag
            else:
                return False,False
    def idname(id):
        with open("users/"+str(id)) as f:
            name = str(f.read())
        return name
    def nameid(name):
        with open("users/"+str(name)+"/id") as f:
            id = int(f.read())
        return id
    def mycon(app):
        con=[]
        for x in console:
            if x[1]==app or x[1]=="all":
                con.append(x)
        return con
def is_socket_closed(sock):
    try:
        sock.send("".encode())
        return False
    except:
        return True
die=False
# Handle Updates
def server():
    while not die:
        for c in range(len(connections)):
            if connections[c]["thread"] and connections[c]["die"]:
                #print(time()-connections[c]["calldelta"])
                if is_socket_closed(connections[c]["socket"]) or time()-connections[c]["calldelta"] > 3:
                    ret = True
                    try:
                        ret = app_disconnect[connections[c]['app']](connections[c])
                        if ret == None:
                            ret = True
                    except:
                        pass
                    connections[c]["thread"]=False
                    console.append([connections[c]["username"] + " disconnected",connections[c]["app"]])
                    connections[c]["username"] = "Guest"
                    connections[c]["auth"] = False
                    connections[c]["app"] = None
                    free.append(c)
                    if ret:
                        print(f"client {connections[c]['id']} disconnected...")
                        logg.log("Client "+connections[c]['id']+" disconnected...")
        #print(connections)
        while len(console)>50:
            del console[0]
        sleep(1)

def stop():
  global die
  die=True
  logg.die=True

app_handle={}
# Handle Clients
def client_handle(cs,c):
    send_all(cs, connections[c]["id"].encode("utf-8"))
    while connections[c]["thread"] and not die:
        try:
            m = recv_all(cs, BUFFER_SIZE)
            if len(m)>0:
                delta=time()-connections[c]["calldelta"]
                connections[c]["calldelta"]=time()
            try:
                d=m.decode()
                string=True
            except:
                d=m
                string=False
            if d == "clientGetID":
                send_all(cs, connections[c]["id"].encode())
            elif d == "bye":
                break
            elif string and d.split(":")[0] == "auth":
                _, username, password,app = d.split(":")
                accept,guest=action.auth(username,password,app)
                if accept and not guest:
                    connections[c]["username"]=username
                    connections[c]["app"]=app
                    connections[c]["auth"]=True
                    send_all(cs, "auth-accepted".encode())
                    console.append(["User "+username+" logged on",app])
                elif accept and guest:
                    no=True
                    for con in connections:
                        if con["username"]==username:
                            send_all(cs, "guest-declined".encode())
                            no=False
                            break
                    if no:
                        connections[c]["username"] = username
                        connections[c]["app"] = app
                        connections[c]["auth"] = True
                        send_all(cs, "guest-accepted".encode())
                        console.append(["Guest " + username + " hopped in",app])
                elif guest and not accept:
                    send_all(cs, "guest-declined".encode())
                else:
                    send_all(cs, "auth-declined".encode())
            else:
                if connections[c]["auth"]:
                    if connections[c]['app'] in app_handle.keys():
                        app_handle[connections[c]['app']](connections[c],d)
                elif d!="" and connections[c]["thread"]:
                    print("message: "+d)
                    send_all(cs, "request-declined".encode())
        except Exception:
            if is_socket_closed(cs):
                break
            else:
                print("client error occurred")
                traceback.print_exc()
            break

    cs.close()

def read():
    global die
    while not die:
        inp = input()
        if inp=="ipbans":
            for ip in ipbans:
                if ip[1]>=10:
                    t=time()-ip[2]
                    pre="seconds"
                    if t>60:
                        t=int(t/60)
                        pre="minutes"
                    if t>60:
                        t=int(t/60)
                        pre="hours"
                    if t>24:
                        t=t/24
                        pre="days"
                    print(ip[0],t,pre)
        elif "unban " in inp:
            i=0
            while i<len(ipbans):
                if ipbans[i][0]==inp.split(" ")[1]:
                    print("unbanned "+ipbans[i][0])
                    del ipbans[i]
                i+=1
            pesave("ipbans",*ipbans)
        elif inp=="kickall":
            c=1
            while c<len(connections):
                if connections[c]["thread"]:
                    connections[c]["thread"]=False
                    console.append([connections[c]["username"] + " disconnected", connections[c]["app"]])
                    connections[c]["username"] = "Guest"
                    connections[c]["auth"] = False
                    connections[c]["app"] = None
                    free.append(c)
                    print(f"client {connections[c]['id']} disconnected...")
                    logg.log("Client " + connections[c]['id'] + " disconnected...")
                c+=1
        elif inp=="users":
            for us in connections:
                if us["thread"]:
                    print("id:",us["id"],",username:",us["username"],",Is guest?",us["guest"],",auth:",us["auth"],",app",us["app"],",die?",us["die"],",room",us["room"])
        elif inp=="freed":
            print(*free)
        elif inp=="help":
            print("""ipbans - Lists all IP bans.
unban <ip> - Removes ban for IP.
kickall - Disconnects all clients.
users - Lists all connections.
freed - Lists all free connection slots.
help - Displays this.""")
        print("~DONE~")

def run(silent=False, log=True):
    global ipbans, logg
    import hexicapi.redlogger as logg
    try:
        ipbans = list(peload("ipbans"))
    except:
        pesave("ipbans", *())
        ipbans = []
    # BIND SOCKET TO SERVICE
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    try:
        s.bind((ip, port))
        print("SERVER running")
    except:
        print("bind failed")
        exit()
    if log:
        logg.log("Connection Successful - " + ip)
    s.listen()
    #
    srthread = threading.Thread(target=server)
    srthread.daemon = True
    srthread.start()
    rthread = threading.Thread(target=web.RUN, args=(ip, webport,True,webpage))
    rthread.daemon = True
    rthread.start()
    #
    readthread = threading.Thread(target=read)
    readthread.daemon = True
    readthread.start()
    # Accept Connections
    while not die:
        cs, ca = s.accept()
        i = 0
        ban = False
        while i < len(ipbans):
            if ipbans[i][0] == ca[0] and ipbans[i][1] >= 10:
                ban = True
                break
            i += 1
        try:
            m = recv_all(cs, BUFFER_SIZE)
            try:
                m.decode()
            except:
                send_all(cs, "welcome".encode("utf-8"))
                if not silent:
                    print("creappy", m)
            if ban:
                send_all(cs, str(ipbans[i][2]).encode())
                cs.close()
            elif m.decode("utf-8") == "stop" and die:
                send_all(cs, "bye!".encode())
                cs.close()
            elif m.decode("utf-8") == "clientGetID":
                id, c = makeID(cs)
                thread = threading.Thread(target=client_handle, args=(cs, c))
                if not silent:
                    print("Client on " + str(ca[0]) + " with ID " + id)
                if log:
                    logg.log("Client on " + str(ca[0]) + " with ID " + id)
                thread.daemon = True
                thread.start()
            elif m.decode("utf-8") == "clientGetID_NODIE":
                id, c = makeID(cs)
                connections[c]["die"] = False
                thread = threading.Thread(target=client_handle, args=(cs, c))
                if not silent:
                    print("Client on " + str(ca[0]) + " with ID " + id)
                thread.daemon = True
                thread.start()
            elif "GET / HTTP/1.1" in m.decode("utf-8"):
                # print("Web interupt")
                cs.send('HTTP/1.0 403 Forbidden\r\n'.encode())
                cs.close()
            else:
                i = 0
                al = True
                ba = 0
                while i < len(ipbans):
                    if ipbans[i][0] == ca[0]:
                        al = False
                        ipbans[i][1] += 1
                        ba = ipbans[i][1]
                        break
                    i += 1
                if al:
                    ipbans.append([ca[0], 1, time()])
                    ba = 1
                pesave("ipbans", *ipbans)
                try:
                    if not silent:
                        print("unknown host attempted connection from IP: " + ca[0] + ", ban counter: " + str(ba))
                    if log:
                        logg.log("unknown host attempted connection from IP: " + ca[0] + ", ban counter: " + str(ba))
                except:
                    if not silent:
                        print("unknown host attempted connection, ban counter: " + str(ba))
                    if log:
                        logg.log("unknown host attempted connection, ban counter: " + str(ba))
                if not silent:
                    print("message: " + m.decode("utf-8"))
                if log:
                    logg.log("message: " + m.decode("utf-8"))
                cs.send(str('please login ' + str(ba) + ' - atm=' + str(
                    time())).encode())
                cs.close()
        except Exception:
            traceback.print_exc()
            pass
    print("server stopping...")
    s.close()

class client:
    def app(f):
        app_handle[f.__name__] = f
    def app_disconnect(f):
        app_disconnect[f.__name__] = f
    def send(client, message):
        try:
            _ = message.decode()
            send_all(client['socket'], message)
        except:
            try:
                send_all(client['socket'], message.encode())
            except:
                send_all(client['socket'], message)
    def receive(client, packet_size = BUFFER_SIZE):
        try:
            m = recv_all(client['socket'], packet_size)
        except:
            return False
        try:
            return m.decode("utf-8")
        except:
            return m
